package com.uas.message.controller;

import com.uas.message.common.domain.MapMessage;
import com.uas.message.common.domain.Page;
import com.uas.message.mail.domain.MailLog;
import com.uas.message.mail.domain.MailReceiver;
import com.uas.message.mail.domain.MailSender;
import com.uas.message.mail.domain.MailTemplate;
import com.uas.message.mail.service.MailReceiverService;
import com.uas.message.mail.service.MailSenderService;
import com.uas.message.mail.service.MailService;
import com.uas.message.mail.service.MailTemplateService;
import com.uas.message.viewmodel.SimpleReceiver;
import com.uas.message.viewmodel.SimpleSender;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * 处理web控制台的邮件及邮件模板等的操作
 * 
 * @author yingp
 *
 */
@Controller
@RequestMapping("/mail")
public class MailController {

	@Autowired
	private MailService mailService;

	@Autowired
	private MailSenderService mailSenderService;

	@Autowired
	private MailReceiverService mailReceiverService;

	@Autowired
	private MailTemplateService mailTemplateService;

	@RequestMapping
	public ModelAndView getHomePage() {
		ModelMap map = new ModelMap();
		map.put("templates", mailTemplateService.findTopUsed(5));
		map.put("senders", mailSenderService.findTopUsed(5));
		map.put("receivers", mailReceiverService.findTopUsed(5));
		// map.put("logs", mailService.findLogs(1, 20));
		return new ModelAndView("mail", map);
	}

	/**
	 * 邮件模板列表
	 * 
	 * @return
	 */
	@RequestMapping(value = "/template", method = RequestMethod.GET)
	public ModelAndView getTemplateListPage() {
		ModelMap map = new ModelMap();
		map.put("templates", mailTemplateService.findAll());
		// TODO
		map.put("userName", "xxx");
		return new ModelAndView("mail_tpl", map);
	}

	/**
	 * 新增、修改模板
	 * 
	 * @param mailSender
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/template", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap saveTemplate(MailTemplate mailTemplate, @RequestBody MultipartFile file) throws IOException {
		if (null != file && !file.isEmpty()) {
			mailTemplate = mailTemplateService.save(mailTemplate, file.getOriginalFilename(), file.getBytes());
		} else {
			mailTemplate = mailTemplateService.save(mailTemplate);
		}
		return new ModelMap("templateId", mailTemplate.getId());
	}

	/**
	 * 查找邮件模板
	 * 
	 * @param templateId
	 * @return
	 */
	@RequestMapping(value = "/template/{templateId}", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getTemplate(@PathVariable("templateId") String templateId) {
		return new ModelMap("template", mailTemplateService.findOne(templateId));
	}

	/**
	 * 删除邮件模板
	 * 
	 * @param templateId
	 */
	@RequestMapping(value = "/template/{templateId}", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteTemplate(@PathVariable("templateId") String templateId) {
		mailTemplateService.delete(templateId);
	}

	/**
	 * 删除邮件模板附件
	 * 
	 * @param templateId
	 * @param path
	 */
	@RequestMapping(value = "/template/attach", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteTemplateAttach(String templateId, String path) {
		mailTemplateService.deleteAttach(templateId, path);
	}

	/**
	 * 新增、修改发送方资料
	 * 
	 * @param mailSender
	 * @return
	 */
	@RequestMapping(value = "/sender", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap saveSender(MailSender mailSender) {
		mailSender = mailSenderService.save(mailSender);
		return new ModelMap("senderId", mailSender.getId());
	}

	/**
	 * 发送者列表
	 * 
	 * @return
	 */
	@RequestMapping(value = "/sender", method = RequestMethod.GET)
	public ModelAndView getSenderListPage() {
		ModelMap map = new ModelMap();
		map.put("senders", mailSenderService.findAll());
		// TODO
		map.put("userName", "xxx");
		return new ModelAndView("mail_sender", map);
	}

	/**
	 * 查找全部发送者
	 * 
	 * @return
	 */
	@RequestMapping(value = "/sender/simpleList", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getSenders() {
		return new ModelMap("senders", SimpleSender.from(mailSenderService.findAll()));
	}

	/**
	 * 查找发送者
	 * 
	 * @param senderId
	 * @return
	 */
	@RequestMapping(value = "/sender/{senderId}", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getSenderPage(@PathVariable("senderId") String senderId) {
		return new ModelMap("sender", mailSenderService.findOne(senderId));
	}

	/**
	 * 删除发送方资料
	 * 
	 * @param senderId
	 */
	@RequestMapping(value = "/sender/{senderId}", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteSender(@PathVariable("senderId") String senderId) {
		mailSenderService.delete(senderId);
	}

	/**
	 * 新增、修改接收方资料
	 * 
	 * @param mailSender
	 * @return
	 */
	@RequestMapping(value = "/receiver", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap saveReceiver(MailReceiver mailReceiver) {
		mailReceiver = mailReceiverService.save(mailReceiver);
		return new ModelMap("receiverId", mailReceiver.getId());
	}

	/**
	 * 接收者列表
	 * 
	 * @return
	 */
	@RequestMapping(value = "/receiver", method = RequestMethod.GET)
	public ModelAndView getReceiverListPage() {
		ModelMap map = new ModelMap();
		map.put("receivers", mailReceiverService.findAll());
		// TODO
		map.put("userName", "xxx");
		return new ModelAndView("mail_receiver", map);
	}

	/**
	 * 查找全部接收者
	 * 
	 * @return
	 */
	@RequestMapping(value = "/receiver/simpleList", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getReceivers() {
		return new ModelMap("receivers", SimpleReceiver.from(mailReceiverService.findAll()));
	}

	/**
	 * 查找接收者
	 * 
	 * @param senderId
	 * @return
	 */
	@RequestMapping(value = "/receiver/{receiverId}", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getReceiverPage(@PathVariable("receiverId") String receiverId) {
		return new ModelMap("receiver", mailReceiverService.findOne(receiverId));
	}

	/**
	 * 删除接收方资料
	 * 
	 * @param senderId
	 */
	@RequestMapping(value = "/receiver/{receiverId}", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteReceiver(@PathVariable("receiverId") String receiverId) {
		mailReceiverService.delete(receiverId);
	}

	/**
	 * 发送邮件 oneToOne
	 * 
	 * @param message
	 * @return
	 */
	@RequestMapping(value = "/send", method = RequestMethod.POST)
	@ResponseBody
	public MailLog sendByTemplate(@RequestBody MapMessage message) {
		return mailService.send(message);
	}

	/**
	 * 发送邮件 oneToMany
	 * 
	 * <pre>
	 * 无模板，使用默认发送方来发送
	 * </pre>
	 * 
	 * @param tos
	 *            接收地址
	 * @param subject
	 *            主题
	 * @param content
	 *            内容
	 * @return
	 */
	@RequestMapping(value = "/send/default", method = RequestMethod.POST)
	@ResponseBody
	public List<Map<String, Object>> sendByDefault(@RequestParam(required = true) String tos, String subject,
                                                   @RequestParam(required = true) String content) {
		return mailService.sendByDefault(tos.split(","), subject, content);
	}

	/**
	 * 发送邮件 oneToMany
	 * 
	 * @param message
	 * @return
	 */
	@RequestMapping(value = "/send/o2m", method = RequestMethod.POST)
	@ResponseBody
	public List<MailLog> sendAllByTemplate(@RequestBody MapMessage message) {
		return mailService.sendAll(message);
	}

	/**
	 * 发送邮件 manyToMany
	 * 
	 * @param templateId
	 * @param receiver
	 * @param params
	 * @return
	 */
	@RequestMapping(value = "/send/m2m", method = RequestMethod.POST)
	@ResponseBody
	public List<MailLog> sendAllByTemplate(@RequestBody List<MapMessage> messages) {
		return mailService.sendAll(messages);
	}

	/**
	 * 查看发送日志
	 * 
	 * @param templateId
	 * @param success
	 * @param pageNumber
	 * @param pageSize
	 * @return
	 */
	@RequestMapping(value = "/send/log", method = RequestMethod.GET)
	@ResponseBody
	public Page<MailLog> getLogs(String templateId, String receiver, Boolean success, int pageNumber, int pageSize) {
		return mailService.findLogs(templateId, receiver, success, pageNumber, pageSize);
	}

	/**
	 * 清除指定模板的发送日志
	 * 
	 * @param templateId
	 * @param pageNumber
	 * @param pageSize
	 * @return
	 */
	@RequestMapping(value = "/send/log", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteLogs(String templateId) {
		mailService.deleteLogs(templateId);
	}

}
