package com.uas.message.controller;

import com.uas.message.common.domain.Page;
import com.uas.message.mq.domain.*;
import com.uas.message.mq.service.MqReceiverService;
import com.uas.message.mq.service.MqSenderService;
import com.uas.message.mq.service.MqService;
import com.uas.message.mq.service.MqTemplateService;
import com.uas.message.viewmodel.SimpleReceiver;
import com.uas.message.viewmodel.SimpleSender;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.List;

/**
 * 处理web控制台的短信及短信模板等的操作
 * 
 * @author huxz
 *
 */
@Controller
@RequestMapping("/mq")
public class MqController {

	private final MqService mqService;

	private final MqSenderService mqSenderService;

	private final MqReceiverService mqReceiverService;

	private final MqTemplateService mqTemplateService;

	@Autowired
	public MqController(MqService mqService, MqSenderService mqSenderService, MqReceiverService mqReceiverService, MqTemplateService mqTemplateService) {
		this.mqService = mqService;
		this.mqSenderService = mqSenderService;
		this.mqReceiverService = mqReceiverService;
		this.mqTemplateService = mqTemplateService;
	}

	@RequestMapping
	public ModelAndView getHomePage() {
		ModelMap map = new ModelMap();
		map.put("templates", mqTemplateService.findTopUsed(5));
		map.put("senders", mqSenderService.findTopUsed(5));
		map.put("receivers", mqReceiverService.findTopUsed(5));
		return new ModelAndView("mq", map);
	}

	/**
	 * 消息模板列表
	 */
	@RequestMapping(value = "/template", method = RequestMethod.GET)
	public ModelAndView getTemplateListPage() {
		ModelMap map = new ModelMap();
		map.put("templates", mqTemplateService.findAll());
		// TODO
		map.put("userName", "xxx");
		return new ModelAndView("mq_tpl", map);
	}

	/**
	 * 新增、修改模板
	 */
	@RequestMapping(value = "/template", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap saveTemplate(MqTemplate mqTemplate) {
		mqTemplate = mqTemplateService.save(mqTemplate);
		return new ModelMap("templateId", mqTemplate.getId());
	}

	/**
	 * 查找消息模板
	 */
	@RequestMapping(value = "/template/{templateId}", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getTemplate(@PathVariable("templateId") String templateId) {
		return new ModelMap("template", mqTemplateService.findOne(templateId));
	}

	/**
	 * 删除消息模板
	 */
	@RequestMapping(value = "/template/{templateId}", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteTemplate(@PathVariable("templateId") String templateId) {
		mqTemplateService.delete(templateId);
	}

	/**
	 * 新增、修改消息推送方资料
	 */
	@RequestMapping(value = "/sender", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap saveSender(MqSender mqSender) {
		mqSender = mqSenderService.save(mqSender);
		return new ModelMap("senderId", mqSender.getId());
	}

	/**
	 * 消息推送者列表
	 */
	@RequestMapping(value = "/sender", method = RequestMethod.GET)
	public ModelAndView getSenderListPage() {
		ModelMap map = new ModelMap();
		map.put("senders", mqSenderService.findAll());
		// TODO
		map.put("userName", "xxx");
		return new ModelAndView("mq_sender", map);
	}

	/**
	 * 查找全部消息推送者
	 */
	@RequestMapping(value = "/sender/simpleList", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getSenders() {
		return new ModelMap("senders", SimpleSender.from(mqSenderService.findAll()));
	}

	/**
	 * 查找消息推送者
	 */
	@RequestMapping(value = "/sender/{senderId}", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getSenderPage(@PathVariable("senderId") String senderId) {
		return new ModelMap("sender", mqSenderService.findOne(senderId));
	}

	/**
	 * 删除消息推送者资料
	 */
	@RequestMapping(value = "/sender/{senderId}", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteSender(@PathVariable("senderId") String senderId) {
		mqSenderService.delete(senderId);
	}

	/**
	 * 新增、修改接收方资料
	 */
	@RequestMapping(value = "/receiver", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap saveReceiver(MqReceiver mqReceiver) {
		mqReceiver = mqReceiverService.save(mqReceiver);
		return new ModelMap("receiverId", mqReceiver.getId());
	}

	/**
	 * 接收者列表
	 */
	@RequestMapping(value = "/receiver", method = RequestMethod.GET)
	public ModelAndView getReceiverListPage() {
		ModelMap map = new ModelMap();
		map.put("receivers", mqReceiverService.findAll());
		// TODO
		map.put("userName", "xxx");
		return new ModelAndView("mq_receiver", map);
	}

	/**
	 * 查找全部接收者
	 */
	@RequestMapping(value = "/receiver/simpleList", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getReceivers() {
		return new ModelMap("receivers", SimpleReceiver.from(mqReceiverService.findAll()));
	}

	/**
	 * 查找接收者
	 */
	@RequestMapping(value = "/receiver/{receiverId}", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getReceiverPage(@PathVariable("receiverId") String receiverId) {
		return new ModelMap("receiver", mqReceiverService.findOne(receiverId));
	}

	/**
	 * 删除接收方资料
	 */
	@RequestMapping(value = "/receiver/{receiverId}", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteReceiver(@PathVariable("receiverId") String receiverId) {
		mqReceiverService.delete(receiverId);
	}

	/**
	 * 发送短信 oneToOne
	 */
	@RequestMapping(value = "/send", method = RequestMethod.POST)
	@ResponseBody
	public MqLog sendByTemplate(@RequestBody MqMessage message) {
		return mqService.send(message);
	}

	/**
	 * 发送短信 oneToMany
	 * 
	 * <pre>
	 * 无模板，使用默认发送方来发送
	 * </pre>
	 *  @param tos
	 *            接收地址
	 * @param content
	 */
	@RequestMapping(value = "/send/default", method = RequestMethod.POST)
	@ResponseBody
	public List<MqLog> sendByDefault(@RequestParam String tos, @RequestParam String content) {
		MessageBody body = new MessageBody();
		body.setContent(content);
		return mqService.sendByDefault(tos.split(","), body);
	}

	/**
	 * 发送短信 oneToMany
	 */
	@RequestMapping(value = "/send/o2m", method = RequestMethod.POST)
	@ResponseBody
	public List<MqLog> sendAllByTemplate(@RequestBody MqMessage message) {
		return mqService.sendAll(message);
	}

	/**
	 * 发送短信 manyToMany
	 */
	@RequestMapping(value = "/send/m2m", method = RequestMethod.POST)
	@ResponseBody
	public List<MqLog> sendAllByTemplate(@RequestBody List<MqMessage> messages) {
		return mqService.sendAll(messages);
	}

	/**
	 * 查看发送日志
	 */
	@RequestMapping(value = "/send/log", method = RequestMethod.GET)
	@ResponseBody
	public Page<MqLog> getLogs(String templateId, String receiver, Boolean success, int pageNumber, int pageSize) {
		return mqService.findLogs(templateId, receiver, success, pageNumber, pageSize);
	}

	/**
	 * 清除指定模板的发送日志
	 */
	@RequestMapping(value = "/send/log", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteLogs(String templateId) {
		mqService.deleteLogs(templateId);
	}

	/**
	 * 查找全部应用名称
	 */
	@RequestMapping(value = "/template/apps", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getApps() {
		return new ModelMap("apps", mqTemplateService.findAllApps());
	}
}
