package com.uas.message.core;

import com.alibaba.fastjson.JSONObject;
import com.uas.dfs.domain.FileInfo;
import com.uas.dfs.domain.MetaData;
import com.uas.dfs.service.FileClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

@Service
public class FileClientImpl implements FileClient {

    private final RestTemplate restTemplate;

    @Autowired
    public FileClientImpl(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    @Override
    public String upload(byte[] bytes, long l, String s, Set<MetaData> set) {
        HttpHeaders headers = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("multipart/form-data;charset=UTF-8");
        headers.setContentType(type);

        File file = new File(s);
        try {
            OutputStream output = new FileOutputStream(file);
            BufferedOutputStream bufferedOutput = new BufferedOutputStream(output);
            bufferedOutput.write(bytes);
        } catch (IOException e) {
            e.printStackTrace();
        }

        MultiValueMap<String, Object> form = new LinkedMultiValueMap<String, Object>();
        ByteArrayResource arrayResource = new ByteArrayResource(bytes){
            @Override
            public String getFilename() throws IllegalStateException {
                return "test." + s;
            }

        };
        form.add("file", arrayResource);

        HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<MultiValueMap<String, Object>>(form, headers);
        ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(FileClientConstant.FILE_UPLOAD, requestEntity, JSONObject.class);
        return responseEntity.getBody().getString("path");
    }

    @Override
    public byte[] download(String s) {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("path", s);
        HttpHeaders headers = new HttpHeaders();
        ResponseEntity<byte[]> response = restTemplate.exchange(FileClientConstant.FILE_DOWNLOAD, HttpMethod.GET,
                new HttpEntity<byte[]>(headers), byte[].class, params);
        return response.getBody();
    }

    @Override
    public void delete(String s) {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("path", s);
        restTemplate.delete(FileClientConstant.FILE_DELETE, params);
    }

    @Override
    public String uploadImage(byte[] bytes, long l, String s, Set<MetaData> set) {
        // 项目无用到，暂不实现
        return null;
    }

    @Override
    public FileInfo getInfo(String s) {
        // 项目无用到，暂不实现
        return null;
    }

    @Override
    public Set<MetaData> getMetaData(String s) {
        // 项目无用到，暂不实现
        return null;
    }

    @Override
    public void mergeMetaData(String s, Set<MetaData> set) {
        // 项目无用到，暂不实现
    }

    @Override
    public void overrideMetaData(String s, Set<MetaData> set) {
        // 项目无用到，暂不实现
    }

}
