var app = {
	loading: function(is) {
		$('#loading').css('display', is ? 'block': 'none');
	},
	info: function(message, title) {
		toastr.info(message, title, {positionClass: 'toast-top-center', timeOut: 1000});
	},
	error: function(message, title) {
		toastr.error(message, title, {positionClass: 'toast-top-center'});
	},
	//============CRUD about sender===============
	editSender : function(event, senderId) {
		senderId = event ? $(event.target).data("id") : senderId;
		if (senderId) {
			$.getJSON('/mq/sender/' + senderId, function(data){
				app.showSenderModal(data.sender);
			});
		} else {
			// new sender
			app.showSenderModal({
				writer: 'HTTP',
				reader: 'JSON'
			});
		}
	},
	getSenders: function(callback) {
		$.getJSON('/mq/sender/simpleList', function(data){
			callback.call(null, data.senders);
		});
	},
	showSenderModal: function(sender) {
		$('#senderModal').showModal(function(){
			var scope = $(this);
			scope.renderData(sender);
			scope.find('.btn-submit').click(scope.find('form'), app.saveSender);
			if(sender.id) {
				scope.find('.btn-del').click(sender, app.deleteSender);
				scope.find('.btn-del').show();
			} else {
				scope.find('.btn-del').hide();
			}
			var toggleWriterType = function(writerType) {
				if('HTTP' == writerType) {
					scope.find('#senderModal_writer_ws').hide();
					scope.find('#senderModal_writer_http').show();
				} else {
					scope.find('#senderModal_writer_http').hide();
					scope.find('#senderModal_writer_ws').show();
				}
			};
			toggleWriterType(sender.writer);
			scope.find(':radio[name=writer]').change(function(){
				toggleWriterType($(this).val());
			});
			var toggleReaderType = function(readerType) {
				if('STRING' == readerType) {
					scope.find('#senderModal_reader_str').show();
				} else {
					scope.find('#senderModal_reader_str').hide();
				}
			};
			toggleReaderType(sender.reader);
			scope.find(':radio[name=reader]').change(function(){
				toggleReaderType($(this).val());
			});
		});
	},
	saveSender : function(event) {
		var form = event.data, sender = form.serializeObject();
		!sender.isDefault && (sender.isDefault = 0);
		// 检验json格式+去掉\n\t等字符
		var json = {};
		if (sender.jsonParams == '')
			sender.jsonParams = '{}';
		if(sender.jsonParams != null && sender.jsonParams.indexOf('{') != 0) {
			var params = sender.jsonParams.split(/\n+|\t+|,|;/);
			$.each(params, function(i, param){
				var kv = param.split('=');
				json[kv[0].trim()] = kv[1].trim();
			});
		} else {
			json = $.parseJSON(sender.jsonParams);
		}
		sender.jsonParams = JSON.stringify(json);
		app.loading(true);
		$.post('/mq/sender', sender, function(data){
			app.loading(false);
			if(data.senderId) {
				window.location.hash = '/sender';
				window.location.reload();
			} else if (data.errMsg) {
				app.error(data.errMsg);
			}
		}, 'json');
	},
	deleteSender: function(event){
		var senderId = event.data.id;
		if(senderId) {
			app.loading(true);
			$.del('/mq/sender/' + senderId, function(data) {
				app.loading(false);
				if (data && data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			});
		}
	},
	//============CRUD about receiver===============
	editReceiver : function(event, receiverId) {
		receiverId = event ? $(event.target).data("id") : receiverId;
		if (receiverId) {
			$.getJSON('/mq/receiver/' + receiverId, function(data){
				app.showReceiverModal(data.receiver);
			});
		} else {
			// new receiver
			app.showReceiverModal({
				userType: 'DEFAULT'
			});
		}
	},
	getReceivers: function(callback) {
		$.getJSON('/mq/receiver/simpleList', function(data){
			callback.call(null, data.receivers);
		});
	},
	showReceiverModal: function(receiver) {
		$('#receiverModal').showModal(function(){
			var scope = $(this);
			scope.renderData(receiver);
			scope.find('.btn-submit').click(scope.find('form'), app.saveReceiver);
			if(receiver.id) {
				scope.find('.btn-del').click(receiver, app.deleteReceiver);
				scope.find('.btn-del').show();
				scope.find('.app-area').hide();
			} else {
				scope.find('.btn-del').hide();
				scope.find('.app-area').show();
			}
			var toggleUserType = function(userType) {
				if('DEFAULT' == userType) {
					scope.find('#receiverModal_user_sql').hide();
					scope.find('#receiverModal_user_default').show();
				} else {
					scope.find('#receiverModal_user_default').hide();
					scope.find('#receiverModal_user_sql').show();
				}
			};
			app.getApps(function (apps) {
				var box = scope.find('#apps-box'), curr_id = receiver.appId;
				box.find('.active').text(curr_id);
				var html = '';
				for (var j in apps) {
					html += '<li><a href="#" title="apps" data-id="' + apps[j] + '">' + apps[j] + '</a></li>'
				}
				box.find('ul').html(html);
				box.find('a').click(function() {
					var that = $(this);
					box.find('.active').text(that.text());
					scope.find('input[name=appId]').val(that.data('id'));
				});
			});
			toggleUserType(receiver.userType);
			scope.find(':radio[name=userType]').change(function(){
				toggleUserType($(this).val());
			});
		});
	},
	saveReceiver : function(event) {
		var form = event.data, receiver = form.serializeObject();
		app.loading(true);
		$.post('/mq/receiver', receiver, function(data){
			app.loading(false);
			if(data.receiverId) {
				window.location.reload();
			} else if (data.errMsg) {
				app.error(data.errMsg);
			}
		}, 'json');
	},
	deleteReceiver: function(event){
		var receiverId = event.data.id;
		if(receiverId) {
			app.loading(true);
			$.del('/mq/receiver/' + receiverId, function(data) {
				app.loading(false);
				if (data && data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			});
		}
	},
	//============CRUD about template===============
	editTemplate : function(event, templateId) {
		templateId = event ? $(event.target).data("id") : templateId;
		if (templateId) {
			$.getJSON('/mq/template/' + templateId, function(data){
				app.showTemplateModal(data.template);
			});
		} else {
			// new template
			app.showTemplateModal({});
		}
	},
	showTemplateModal: function(template) {
		$('#templateModal').showModal({
			show: function() {
				var scope = $(this), editTab = scope.find('#templateModal_edit');
				editTab.renderData(template);
				if(template.id) {
					editTab.find('.btn-del').click(template, app.deleteTemplate);
					editTab.find('.btn-del').show();
					editTab.find('.defineType-area').hide();
					editTab.find('.app-area').hide();
				} else {
					editTab.find('.btn-del').hide();
					editTab.find('.defineType-area').show();
					editTab.find('.app-area').show();
					scope.find('a[data-toggle="tab"]').hide();
				}
			},
			shown: function(){
				var scope = $(this), editTab = scope.find('#templateModal_edit');
				editTab.find('.btn-submit').click({
					element: editTab.find('form')
				}, app.saveTemplate);
				// load senders
				app.getSenders(function(senders){
					var box = editTab.find('#senders-box'), curr_id = template.senderId;
					if(curr_id) {
						for(var i in senders) {
							if(senders[i].id == curr_id) {
								box.find('.active').text(senders[i].description);
								break;
							}
						}
					}
					box.find('ul').appendTpl('<li><a href="#" title="#{username}" data-id="#{id}">#{description}</a></li>', senders);
					box.find('a').click(function() {
						var that = $(this);
						box.find('.active').text(that.text());
						editTab.find('input[name=senderId]').val(that.data('id'));
					});
				});
				app.getApps(function (apps) {
					var box = editTab.find('#apps-box'), curr_id = template.appId;
					if(curr_id) {
						for(var i in apps) {
							if(apps[i] == curr_id) {
								box.find('.active').text(apps[i]);
								break;
							}
						}
					}
					var html = '';
					for (var j in apps) {
						html += '<li><a href="#" title="apps" data-id="' + apps[j] + '">' + apps[j] + '</a></li>'
					}
					box.find('ul').html(html);
					box.find('a').click(function() {
						var that = $(this);
						box.find('.active').text(that.text());
						editTab.find('input[name=appId]').val(that.data('id'));
					});
				});
				var showTypes = function() {
					var box = editTab.find('#types-box'), curr_type = template.defineType;
					if (curr_type) {
						box.find('.active').text(curr_type);
					}
					var html = '';
					var types = ['SYSTEM', 'CUSTOM'];
					for (var i in types) {
						html += '<li><a href="#" title="types" data-id="' + types[i] + '">' + types[i] + '</a></li>'
					}
					box.find('ul').html(html);
					box.find('a').click(function() {
						var that = $(this);
						box.find('.active').text(that.text());
						editTab.find('input[name=defineType]').val(that.data('id'));
					});
				};
				showTypes();
				// listen on tab active
				scope.find('a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
					var tabId = $(e.target).attr('aria-controls'), tab = scope.find("#" + tabId),
						ready = tab.attr('viewready');
					if(!ready) {
						tab.attr('viewready', true);
						if(tabId == 'templateModal_log') {
							app.showSendLog({
								templateId: template.id,
								pageNumber: 1,
								pageSize: 10
							});
							tab.find('.btn-clearlog').click(template, app.deleteSendLog);
						} else if(tabId == 'templateModal_send') {
							app.getReceivers(function(receivers){
								var box = tab.find("#receivers-box");
								box.appendTpl('<label class="checkbox-inline"><input type="checkbox" name="receiverIds" value="#{id}"> #{description}</label>', receivers);
							});
							tab.find('.btn-send').click({
								element: tab.find('form'),
								resolve: function() {
									return {
										templateId: template.id
									};
								}
							}, app.sendMq);
						}
					}
				});
			}
		});
	},
	getApps: function(callback) {
		$.getJSON('/mq/template/apps', function(data){
			callback.call(null, data.apps);
		});
	},
	saveTemplate : function(event) {
		var form = event.data.element, template = form.serializeObject();
		app.loading(true);
		$.post('/mq/template', template, function(data){
			app.loading(false);
			if(data.templateId) {
				window.location.reload();
			} else if (data.errMsg) {
				app.error(data.errMsg);
			}
		}, 'json');
	},
	deleteTemplate: function(event){
		var templateId = event.data.id;
		if(templateId) {
			app.loading(true);
			$.del('/mq/template/' + templateId, function(data) {
				app.loading(false);
				if (data && data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			});
		}
	},
	sendMq: function(event) {
		var form = event.data.element, mq = $.extend(form.serializeObject(), event.data.resolve.call());
		mq.receivers = (mq.receivers ? (mq.receivers.split(/\n+|\t+|\s+|,|;/)) : null);
		mq.params = (mq.params || '').split('\n');
		if (typeof mq.receiverIds == 'string') {
			mq.receiverIds = [mq.receiverIds];
		}
		mq.body = {
			title : mq.title,
			pageTitle : mq.pageTitle,
			url : mq.url,
			content : mq.content,
			platform : mq.platform
		};
		app.loading(true);
		$.ajax({
			url: '/mq/send/o2m',
			type: 'POST',
			contentType:'application/json; charset=UTF-8',
			data: JSON.stringify(mq),
			processData: false,
			dataType: 'json',
			success: function(data){
				app.loading(false);
				console.log(arguments);
				if (data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			},
			error:  function(data){
				app.loading(false);
				if (data.responseJSON && data.responseJSON.errMsg) {
					app.error(data.responseJSON.errMsg);
				} else {
					window.location.reload();
				}
			}
		});
	},
	//============about log===============
	showSendLog: function(params) {
		$('.pagination').pagination({
			url: '/mq/send/log',
			params: params,
			renderData: function(content){
				var list = $('.x-log-list .x-common-list');
				if(content.length == 0) {
					list.empty();
					$('.x-log-list .x-empty').show();
				} else {
					$('.x-log-list .x-empty').hide();
					$.each(content, function(i, d){
						d.sendDate = new Date(d.sendDate).format('yy-mm-dd h:i');
					});
					list.appendTpl('<div class="x-item"><div class="x-icon"><i class="fa fa-envelope-o"></i></div><p><span class="x-title text-info">#{description}</span><span class="pull-right x-text-muted">#{sendDate}</span></p><div class="x-text-muted">#{sender} <i class="fa fa-fw fa-mall-forward"></i> #{receiver}#{fn:showStatus}</div></div>', content, {
						showStatus: function(data) {
							if (data.success) {
								return '<span class="pull-right" title="发送成功"><i class="fa fa-check"></i></span>';
							} else if (data.errMsg) {
								return '<span class="pull-right" data-toggle="popover" data-trigger="hover" data-placement="left" data-content="' + data.errMsg + '"><i class="fa fa-times"></i></span>';
							}
						}
					});
					$('[data-toggle="popover"]').popover();
				}
			}
		});
	},
	searchSendLog: function(event) {
		var target = $(event.target), form = $('.x-nav-tabs form'), data = form.serializeObject();
		if (!target.is('a')) {
			target = $('.x-nav-tabs .active a');
		}
		data.success = target.data('success');
		data.pageNumber = 1;
		app.showSendLog(data);
		$('.x-nav-tabs .active').removeClass('active');
		target.parent().addClass('active');
	},
	deleteSendLog: function(event) {
		var templateId = event.data.id;
		if(templateId) {
			app.loading(true);
			$.del('/mq/send/log', {
				templateId: templateId
			}, function(data) {
				app.loading(false);
				if (data && data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			});
		}
	},
	//============document ready===============
	init : function() {
		$('.x-sender-list .x-editor-trigger').click(app.editSender);
		$('.x-receiver-list .x-editor-trigger').click(app.editReceiver);
		$('.x-tpl-list .x-editor-trigger').click(app.editTemplate);
		new Router({
			'#/home': function() {
				app.showSendLog();
				$('.x-nav-tabs a').click(app.searchSendLog);
				$('.x-nav-tabs .btn-search').click(app.searchSendLog);
			},
			'#/template/(.*)': function(id) {
				app.editTemplate(null, id);
			},
			'#/sender/(.*)': function(id) {
				app.editSender(null, id);
			},
			'#/receiver/(.*)': function(id) {
				app.editReceiver(null, id);
			}
		}).start();
		app.loading(false);
	}
};
$(document).ready(function() {
	app.init();
});